/*
CartoTypeLegend.h
Copyright 2015-2021 CartoType Ltd.
See www.cartotype.com for more information.
*/

#ifndef CartoTypeLegend_h__
#define CartoTypeLegend_h__

#import <CartoType/CartoTypeBase.h>
#import <QuartzCore/QuartzCore.h>

/**
\file
The CartoTypeLegend class.
*/

@class CartoTypeFramework; // forward declaration

/**
The CartoTypeLegend class specifies legends or map keys showing sample map objects, with optional text lines and a scale.
 
Dimensions are specified with units, which may be "pt" (point), "pc" (pica), "cm" (centimetre),
"mm" (millimetre), "in" (inch), or "px" (pixel). An empty or unrecognised unit is taken as "px".
 
To use a legend on a map, create a Legend object and pass it to the setLegend method of CartoTypeFramework.
*/
@interface CartoTypeLegend: NSObject

/** Initializes a CartoTypeLegend with a CartoTypeFramework object, using the same fonts and style sheet as aFramework. */
-(id)initWithFramework:(CartoTypeFramework*)aFramework;

/** Destroys the object. */
-(void)dealloc;

/** Deletes all legend lines (map objects, text lines and scale bars). */
-(void)clear;

/**
Adds a line containing a map object, with an optional label to the left of it.
If the parameter aOsmType is a type code made from three lower-case letters in the range a...z, it is converted to an integer type code which is ORed with aIntAttrib.
*/
-(void)addMapObjectLine:(CartoTypeMapObjectType)aType layer:(NSString*)aLayer freatureInfo:(CartoTypeFeatureInfo*)aFeatureInfo stringAttrib:(NSString*)aStringAttrib label:(NSString*)aLabel;

/** Adds a line of text, using the current font family, font size, text color and minimum line height. */
-(void)addTextLine:(NSString*)aText;

/**
Adds a scale bar line, using the current font family, font size, alignment, text color and minimum line height.
If aMetricUnits is true kilometers and meters are used, otherwise miles and yards.
*/
-(void)addScaleLine:(bool)aMetricUnits;

/** Sets the main style sheet, replacing the style sheet copied from the Framework parameter to the Legend constructor. */
-(void)setMainStyleSheet:(NSString*)aData;

/**
Sets the extra style sheet.
 
The purpose of this function is to allow a caller such as a style sheet editor
to create a style sheet containing a single layer, or a small number of
layers, and draw a legend for that layer only.
*/
-(void)setExtraStyleSheet:(NSString*)aData;

/** Sets the background color. Transparent colors are allowed. */
-(void)setBackgroundColor:(CartoTypeColor)aColor;

/** Sets the border color and corner radius for the whole legend. Transparent border colors are allowed. */
-(void)setBorderColor:(CartoTypeColor)aColor strokeWidth:(double)aStrokeWidth radius:(double)aRadius unit:(NSString*)aUnit;

/** Sets the margin width for the whole legend. */
-(void)setMarginWidth:(double)aMarginWidth unit:(NSString*)aUnit;

/** Sets the minimum line height for subsequently added lines. */
-(void)setMinLineHeight:(double)aLineHeight unit:(NSString*)aUnit;

/** Sets the label wrap width for subsequently added lines. Values of zero or less prevent wrapping. */
-(void)setLabelWrapWidth:(double)aWrapWidth unit:(NSString*)aUnit;

/** Sets the font family for subsequently added lines. */
-(void)setFontFamily:(NSString*)aFontFamily;

/** Sets the font size for subsequently added lines. */
-(void)setFontSize:(double)aFontSize unit:(NSString*)aUnit;

/** Sets the text color for subsequently added lines. */
-(void)setTextColor:(CartoTypeColor)aTextColor;

/** Sets the diagram color, used for scale bars and turn diagrams, for subsequently added lines. */
-(void)setDiagramColor:(CartoTypeColor)aDiagramColor;

/** Sets the alignment for labels and scale bars. */
-(void)setAlignment:(CartoTypeAlign)aAlignment;

/** Sets the amount in degrees by which rectangles drawn for polygon legend lines are rotated, for subsequently added lines. */
-(void)setPolygonRotation:(double)aDegrees;

/** Returns true if any of the lines of this legend object is a turn instruction. */
-(bool)hasTurnInstruction;

/**
Sets the instruction to be displayed for the current turn, overriding automatically generated instructions.
To revert to automatically generated instructions, call this function with empty text.
*/
-(void)setTurnInstruction:(NSString*)aText;

/**
Returns the instruction to be displayed for the current turn, overriding automatically generated instructions.
Returns the empty string if automatically generated instructions are used and there is no overriding instruction.
*/
-(NSString*)turnInstruction;

/** Gets a pointer to the C++ CLegend object: internal use only. */
-(void*)legend;

@end

#endif
